-- alerter allows users to set key phrases to watch for in text and then generates a large, flashy message to warn them
-- todo:

-- add an "Enabled" checkbox per alert

-- French and German translations

import "GaranStuff.Alerter.Class";
import "GaranStuff.Alerter.VindarPatch";
import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "GaranStuff.Alerter.Table";
import "GaranStuff.Alerter.Strings";
import "GaranStuff.Alerter.ColorPicker";
import "GaranStuff.Alerter.DropDownList";

FlasherFont=Turbine.UI.Lotro.Font.TrajanProBold36;
FlasherColor=Turbine.UI.Color(1,0,0);
AlerterLeft=-1;
AlerterTop=-1;
alertHUDState=true;
alertMainState=false;
alertLogState=false;

fontFace=Turbine.UI.Lotro.Font.Verdana16;
trimColor=Turbine.UI.Color(.3,.3,.3);
fontColor=Turbine.UI.Color(1,1,1);
backColor=Turbine.UI.Color(0,0,0);
listTextColor=Turbine.UI.Color(.9,.9,.9);

logChat=false;

language=1;
locale = "en";
if Turbine.Shell.IsCommand("hilfe") then
	locale = "de";
	language=3;
elseif Turbine.Shell.IsCommand("aide") then
	locale = "fr";
	language=2;
end
euroFormat=(tonumber("1,000")==1);

string.split=function(str,separator)
	local tmpArray={};
	local tmpElem;
	str=tostring(str);
	if string.find("^$()%.[]*+-?",separator,1,true)~=nil then separator="%"..separator end
	for tmpElem in string.gmatch(str,"[^"..separator.."]*") do
		table.insert(tmpArray,tmpElem);
	end
	local tmpIndex=1;
	while tmpIndex<=#tmpArray do
		if tmpIndex==#tmpArray and tmpArray[tmpIndex]=="" then
			table.remove(tmpArray,tmpIndex);
		else
			if tmpArray[tmpIndex]=="" and tmpArray[tmpIndex+1]~="" then
				table.remove(tmpArray,tmpIndex);
			else
				tmpIndex=tmpIndex+1;
			end
		end
	end
	return tmpArray;
end
string.ltrim=function(str)
	return (string.gsub(str, "^%s*(.-)", "%1"))
end
string.rtrim=function(str)
	return (string.gsub(str, "^(.-)%s*$", "%1"))
end
string.trim=function(str)
	return (string.gsub(str, "^%s*(.-)%s*$", "%1"))
end

function AddCallback(object, event, callback)
    if (object[event] == nil) then
        object[event] = callback;
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], callback);
        else
            object[event] = {object[event], callback};
        end
    end
    return callback;
end
function RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil;
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);
            for i = 1, size do
                if (object[event][i] == callback) then
                    table.remove(object[event], i);
                    break;
                end
            end
        end
    end
end

function UnloadPlugin()
	-- have to remove all shell commands before unloading or we get a crash to desktop :(
	alertMain:RemoveShellCommands();
	SaveData();
	RemoveCallback(Turbine.Chat, "Received", alertMain.alerterReceived);
	Turbine.Shell.WriteLine("Alerter "..Plugins["Alerter"]:GetVersion().." by Garan unloaded");
end

function SaveData()
	local settings={};
--	settings[#settings+1]={"MapHeight",mapWindow:GetHeight()};
	PatchDataSave( Turbine.DataScope.Character, "Alerter_Settings", Settings );
	PatchDataSave( Turbine.DataScope.Character, "Alerter_Alerts", Alerts );
end
function LoadData()
	local tmpIndex;
	Alerts=PatchDataLoad( Turbine.DataScope.Character, "Alerter_Alerts");
	if Alerts==nil then Alerts={} end
	for tmpIndex=1,#Alerts do
		if Alerts[tmpIndex][3]~=nil then
			if euroFormat then
				-- replace "." with ","
				Alerts[tmpIndex][3]=tonumber((string.gsub(Alerts[tmpIndex][3],"%.",",")));
			else
				Alerts[tmpIndex][3]=tonumber((string.gsub(Alerts[tmpIndex][3],",",".")));
			end
		end
		if Alerts[tmpIndex][4]~=nil then
			if euroFormat then
				Alerts[tmpIndex][4]=Turbine.UI.Color(tonumber((string.gsub(Alerts[tmpIndex][4].R,"%.",","))),tonumber((string.gsub(Alerts[tmpIndex][4].G,"%.",","))),tonumber((string.gsub(Alerts[tmpIndex][4].B,"%.",","))));
			else
				Alerts[tmpIndex][4]=Turbine.UI.Color(tonumber((string.gsub(Alerts[tmpIndex][4].R,",","."))),tonumber((string.gsub(Alerts[tmpIndex][4].G,",","."))),tonumber((string.gsub(Alerts[tmpIndex][4].B,",","."))));
			end
		end
		if Alerts[tmpIndex][6]~=nil then
			if euroFormat then
				Alerts[tmpIndex][6]=tonumber((string.gsub(Alerts[tmpIndex][6],"%.",",")));
			else
				Alerts[tmpIndex][6]=tonumber((string.gsub(Alerts[tmpIndex][6],",",".")));
			end
		end
		if Alerts[tmpIndex][9]~=nil then
			if euroFormat then
				Alerts[tmpIndex][9]=tonumber((string.gsub(Alerts[tmpIndex][9],"%.",",")));
			else
				Alerts[tmpIndex][9]=tonumber((string.gsub(Alerts[tmpIndex][9],",",".")));
			end
		end
		Alerts[tmpIndex][10]=0; -- set the "next allowed time" to 0
		if Alerts[tmpIndex][11]~=nil then
			if euroFormat then
				Alerts[tmpIndex][11]=tonumber((string.gsub(Alerts[tmpIndex][11],"%.",",")));
			else
				Alerts[tmpIndex][11]=tonumber((string.gsub(Alerts[tmpIndex][11],",",".")));
			end
		end
		if Alerts[tmpIndex][12]~=nil then
			if euroFormat then
				Alerts[tmpIndex][12]=tonumber((string.gsub(Alerts[tmpIndex][12],"%.",",")));
			else
				Alerts[tmpIndex][12]=tonumber((string.gsub(Alerts[tmpIndex][12],",",".")));
			end
		end
		if Alerts[tmpIndex][13]~=nil then
			if euroFormat then
				Alerts[tmpIndex][13]=tonumber((string.gsub(Alerts[tmpIndex][13],"%.",",")));
			else
				Alerts[tmpIndex][13]=tonumber((string.gsub(Alerts[tmpIndex][13],",",".")));
			end
		end
		if Alerts[tmpIndex][14]~=nil then
			if euroFormat then
				Alerts[tmpIndex][14]=tonumber((string.gsub(Alerts[tmpIndex][14],"%.",",")));
			else
				Alerts[tmpIndex][14]=tonumber((string.gsub(Alerts[tmpIndex][14],",",".")));
			end
		end
	end
end
LoadData();


alertTemplate=Turbine.UI.Window();
alertTemplate:SetBackground("GaranStuff/Alerter/Resources/Border.jpg");
alertTemplate.Middle=Turbine.UI.Control();
alertTemplate.Middle:SetParent(alertTemplate);
alertTemplate.Middle:SetBackColor(Turbine.UI.Color(0,0,0,0));
alertTemplate.Middle:SetPosition(10,10);
alertTemplate.SizeChanged=function()
	local width,height=alertTemplate:GetSize();
	alertTemplate.Middle:SetSize(width-20,height-20);
end
alertTemplate.MoveX=-1;
alertTemplate.MoveY=-1;
alertTemplate.MovingIcon=Turbine.UI.Control();
alertTemplate.MovingIcon:SetParent(alertTemplate);
alertTemplate.MovingIcon:SetSize(32,32);
alertTemplate.MovingIcon:SetBackground(0x410081c0)
alertTemplate.MovingIcon:SetStretchMode(2);
-- for some reason, setting the control's position or size AFTER loading the resource background makes it appear... WEIRD
alertTemplate.MovingIcon:SetPosition(alertTemplate:GetWidth()/2-15,alertTemplate:GetHeight()-21);
alertTemplate.MovingIcon:SetVisible(false);
alertTemplate.MouseDown=function(sender,args)
	if (args.Y>alertTemplate:GetHeight()-10) and (args.X>alertTemplate:GetWidth()-10) then
		alertTemplate.MoveX=args.X;
		alertTemplate.MoveY=args.Y;
		alertTemplate.MovingIcon:SetLeft(args.X-12);
		alertTemplate.MovingIcon:SetTop(args.Y-12);
		alertTemplate.MovingIcon:SetSize(32,32);
		alertTemplate.MovingIcon:SetBackground(0x41007e20)
		alertTemplate.MovingIcon:SetVisible(true);
	elseif (args.Y>alertTemplate:GetHeight()-10) then
		alertTemplate.MoveY=args.Y;
		alertTemplate.MovingIcon:SetLeft(args.X-22);
		alertTemplate.MovingIcon:SetTop(args.Y-12);
		alertTemplate.MovingIcon:SetSize(32,32);
		alertTemplate.MovingIcon:SetBackground(0x410081c0)
		alertTemplate.MovingIcon:SetVisible(true);
	elseif (args.X>alertTemplate:GetWidth()-10) then
		alertTemplate.MoveX=args.X;
		alertTemplate.MovingIcon:SetLeft(args.X-12);
		alertTemplate.MovingIcon:SetTop(args.Y-22);
		alertTemplate.MovingIcon:SetSize(32,32);
		alertTemplate.MovingIcon:SetBackground(0x410081bf)
		alertTemplate.MovingIcon:SetVisible(true);
	elseif (args.Y<11) then
		alertTemplate.MoveX=args.X;
		alertTemplate.MoveY=args.Y;
		alertTemplate.MovingIcon:SetLeft(args.X-12);
		alertTemplate.MovingIcon:SetTop(args.Y-12);
		alertTemplate.MovingIcon:SetSize(32,32);
		alertTemplate.MovingIcon:SetBackground(0x410000dd)
		alertTemplate.MovingIcon:SetVisible(true);
	else
		alertTemplate.MoveX=-1;
		alertTemplate.MoveY=-1;
	end
end

alertTemplate.MouseMove=function(sender,args)
	if (alertTemplate.MoveY>-1 and alertTemplate.MoveY<10) then
		if (args.X~=alertTemplate.MoveX or args.Y~= alertTemplate.MoveY) then
			alertTemplate.Moving=true;
			local newLeft=alertTemplate:GetLeft()-(alertTemplate.MoveX-args.X)
			local newTop=alertTemplate:GetTop()-(alertTemplate.MoveY-args.Y)
			if newLeft<0 then newLeft=0 end;
			if newLeft>(Turbine.UI.Display.GetWidth()-alertTemplate:GetWidth()) then newLeft=Turbine.UI.Display.GetWidth()-alertTemplate:GetWidth() end;
			if newTop<0 then newTop=0 end;
			if newTop>(Turbine.UI.Display.GetHeight()-alertTemplate:GetHeight()) then newTop=Turbine.UI.Display.GetHeight()-alertTemplate:GetHeight() end;
			alertTemplate:SetPosition(newLeft,newTop);
			alertMain.LeftText:SetText(newLeft);
			alertMain.LeftPercent:SetText(tostring(math.floor(newLeft/Turbine.UI.Display:GetWidth()*1000+.5)/10).."%")
			alertMain.TopText:SetText(newTop);
			alertMain.TopPercent:SetText(tostring(math.floor(newTop/Turbine.UI.Display:GetHeight()*1000+.5)/10).."%")
		end
	else		
		if (alertTemplate.MoveY>-1) then
			if args.Y~= alertTemplate.MoveY then
				local newHeight=alertTemplate:GetHeight()-(alertTemplate.MoveY-args.Y);
				if newHeight>(Turbine.UI.Display.GetHeight()-alertTemplate:GetTop()) then newHeight=Turbine.UI.Display.GetHeight()-alertTemplate:GetTop() end;
				local newX=args.X-22;
				if newX<-13 then newX=-13 end
				if newX>(alertTemplate:GetWidth()-18) then newX=alertTemplate:GetWidth()-18 end
				alertTemplate.MovingIcon:SetLeft(newX);
				alertTemplate.MovingIcon:SetTop(newHeight-21)
				alertTemplate:SetHeight(newHeight);
				alertTemplate.MoveY=args.Y;
				alertMain.HeightText:SetText(newHeight);
				alertMain.HeightPercent:SetText(tostring(math.floor(newHeight/Turbine.UI.Display:GetHeight()*1000+.5)/10).."%")
			end
		end
		if (alertTemplate.MoveX>-1) then
			if args.X~= alertTemplate.MoveX then
				local newWidth=alertTemplate:GetWidth()-(alertTemplate.MoveX-args.X);
				if newWidth>(Turbine.UI.Display.GetWidth()-alertTemplate:GetLeft()) then newWidth=Turbine.UI.Display.GetWidth()-alertTemplate:GetLeft() end;
				local newY=args.Y-22;
				if newY<-13 then newY=-13 end
				if newY>(alertTemplate:GetHeight()-18) then newY=alertTemplate:GetHeight()-18 end
				alertTemplate.MovingIcon:SetTop(newY);
				alertTemplate.MovingIcon:SetLeft(newWidth-21);
				alertTemplate:SetWidth(newWidth);
				alertTemplate.MoveX=args.X;
				alertMain.WidthText:SetText(newWidth);
				alertMain.WidthPercent:SetText(tostring(math.floor(newWidth/Turbine.UI.Display:GetWidth()*1000+.5)/10).."%")
			end
		end
	end
end
alertTemplate.MouseUp=function(sender,args)
	alertTemplate.MovingIcon:SetVisible(false);
	alertTemplate.MoveX=-1;
	alertTemplate.MoveY=-1;
end

alertTemplate:SetSize(Turbine.UI.Display:GetWidth(),Turbine.UI.Display:GetHeight());

alertLog=Turbine.UI.Lotro.Window();
alertLog:SetBackColor(Turbine.UI.Color(0,0,0));
alertLog:SetText(Resource[language][51]);
alertLog:SetSize(640,480);
alertLog:SetPosition(0,0); -- we can revise this later
alertLog.History=Turbine.UI.ListBox();
alertLog.History:SetParent(alertLog);
alertLog.History:SetSize(608,400);
alertLog.History:SetPosition(10,45);
alertLog.History:SetBackColor(Turbine.UI.Color(.05,.05,.1));
alertLog.VScroll=Turbine.UI.Lotro.ScrollBar();
alertLog.VScroll:SetOrientation(Turbine.UI.Orientation.Vertical);
alertLog.VScroll:SetParent(alertLog);
alertLog.VScroll:SetBackColor(backColor);
alertLog.VScroll:SetPosition(alertLog:GetWidth()-22,45);
alertLog.VScroll:SetWidth(12);
alertLog.VScroll:SetHeight(400);
alertLog.History:SetVerticalScrollBar(alertLog.VScroll);
alertLog.StartButton=Turbine.UI.Lotro.Button();
alertLog.StartButton:SetParent(alertLog);
alertLog.StartButton:SetSize(100,20);
alertLog.StartButton:SetPosition(40,alertLog:GetHeight()-30)
alertLog.StartButton:SetText(Resource[language][57]);
alertLog.StartButton.MouseClick=function()
	logChat=not logChat;
	if logChat then
		alertLog.StartButton:SetText(Resource[language][58])
	else
		alertLog.StartButton:SetText(Resource[language][57])
	end
end
alertLog.ClearButton=Turbine.UI.Lotro.Button();
alertLog.ClearButton:SetParent(alertLog);
alertLog.ClearButton:SetSize(100,20);
alertLog.ClearButton:SetPosition(alertLog:GetWidth()-alertLog.ClearButton:GetWidth()-40,alertLog:GetHeight()-30)
alertLog.ClearButton:SetText(Resource[language][52]);
alertLog.ClearButton.MouseClick=function()
	alertLog.History:ClearItems();
end
alertLog.tmpText=Turbine.UI.Label();
alertLog.tmpText:SetParent(alertLog);
alertLog.tmpText:SetPosition(alertLog:GetWidth()+10,0);
alertLog.tmpVScroll=Turbine.UI.Lotro.ScrollBar();
alertLog.tmpVScroll:SetOrientation(Turbine.UI.Orientation.Vertical);
alertLog.tmpVScroll:SetParent(alertLog);
alertLog.tmpVScroll:SetBackColor(backColor);
alertLog.tmpVScroll:SetPosition(0,-11);
alertLog.tmpVScroll:SetWidth(12);
alertLog.tmpVScroll:SetHeight(14);
alertLog.tmpText:SetVerticalScrollBar(alertLog.tmpVScroll);

Flasher=class(Turbine.UI.Window);

function Flasher:Constructor()
	Turbine.UI.Window.Constructor( self );
	self:SetMouseVisible(false);
	self:SetSize(Turbine.UI.Display:GetWidth(),Turbine.UI.Display:GetHeight());
	self.Opacity=.5;
	self.Image=Turbine.UI.Control();
	self.Image:SetParent(self);
	self.Image:SetMouseVisible(false);
	self.Message=Turbine.UI.Label();
	self.Message:SetParent(self);
	self.Message:SetSize(self:GetWidth(),self:GetHeight());
	self.Message:SetFont(FlasherFont);
	self.Message:SetForeColor(FlasherColor);
	self.Message:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	self.Message:SetMouseVisible(false);
	self.Message:SetMultiline(true);
	self.Message:SetStretchMode(3);
	self.Duration=1;
	self.Interval=.3;

	self:SetZOrder(1000);
	self.EndTime=0;
	self.SetAlert=function(msg, duration, color, font, interval, left, top, width, height, image, captures, channel, sender, origmessage)
		if captures~=nil and type(captures)=="table" then
			-- replace the %x values with the captures
			local tmpCapture;
			for tmpCapture=1,#captures do
				msg=string.gsub(msg,"(%%"..tostring(tmpCapture)..")",captures[tmpCapture]);
			end
		end
		if channel~=nil then
			msg=string.gsub(msg,"(%%C)",channel);
		end
		if sender~=nil then
			msg=string.gsub(msg,"(%%S)",sender);
		end
		if origmessage~=nil then
			msg=string.gsub(msg,"(%%M)",origmessage);
		end
		alertTemplate:SetVisible(false);
		if color~=nil then
			self.Message:SetForeColor(color);
		else
			self.Message:SetForeColor(FlasherColor);
		end
		if interval~=nil then
			self.Interval=interval;
		else
			self.Interval=.3;
		end
		if font~=nil then
			self.Message:SetFont(font);
		else
			self.Message:SetFont(FlasherFont);
		end
		if duration==nil then
			duration=1;
		end
		top=tonumber(top);
		left=tonumber(left);
		width=tonumber(width);
		height=tonumber(height);
		if top==nil or top<0 then top=0 end
		if top>100 then top=100 end
		if left==nil or left<0 then left=0 end
		if left>100 then left=100 end
		if width==nil or width>100 then width=100 end
		if width<0 then width=0 end
		if height==nil or height>100 then height=100 end
		if height<0 then height=0 end

		self:SetSize(Turbine.UI.Display:GetWidth()*width/100,Turbine.UI.Display:GetHeight()*height/100);
		self:SetPosition(Turbine.UI.Display:GetWidth()*left/100,Turbine.UI.Display:GetHeight()*top/100);
		self.Image:SetStretchMode(0);
		if image~=nil and image~="" then
			self.Image:SetVisible(true);
			self.Image:SetSize(1,1);
			if image==tonumber(image) then
				local success,result=pcall(Turbine.UI.Control.SetBackground,self.Image,image);
			else
				local success,result=pcall(Turbine.UI.Control.SetBackground,self.Image,"GaranStuff/Alerter/Resources/"..image);
			end
			self.Image:SetStretchMode(2);
			local width,height=self.Image:GetSize();
			self.Image:SetSize(width,height);
			self.Image:SetStretchMode(1);
			self.Image:SetSize(self:GetWidth(),self:GetHeight());
		else
			self.Image:SetVisible(false);
		end
		self.Message:SetSize(self:GetWidth(),self:GetHeight());
		self.Message:SetText(msg);
		self.EndTime=Turbine.Engine.GetGameTime()+duration;
		if self.Interval==0 then
			self.FlashTime=self.EndTime
		else
			self.FlashTime=Turbine.Engine.GetGameTime()+self.Interval/2;
		end
		self:SetVisible(true);
		self:SetWantsUpdates(true);
	end
	self.Update=function()
		if Turbine.Engine.GetGameTime()>=self.EndTime then
			self:SetWantsUpdates(false);
			self:SetVisible(false);
			self.FlashTime=0;
			alertTemplate:SetVisible(alertMain:IsVisible());
		elseif Turbine.Engine.GetGameTime()>=self.FlashTime then
			self:SetVisible(not self:IsVisible());
			self.FlashTime=Turbine.Engine.GetGameTime()+self.Interval/2;
		end
	end
end

Alerter=class(Turbine.UI.Lotro.Window);
function Alerter:Constructor()
	local tmpIndex;
	Turbine.UI.Lotro.Window.Constructor( self );
	self:SetSize(550,600);
	if AlerterLeft<0 then AlerterLeft=(Turbine.UI.Display:GetWidth()-self:GetWidth())/2 end
	if AlerterTop<0 then AlerterTop=(Turbine.UI.Display:GetHeight()-self:GetHeight())/2 end
	self:SetPosition(AlerterLeft,AlerterTop);
	self:SetText(Resource[language][2])
	self.Flasher=Flasher();
	self:SetZOrder(1);

	self.LanguageCaption=Turbine.UI.Label();
	self.LanguageCaption:SetParent(self);
	self.LanguageCaption:SetPosition(10,45);
	self.LanguageCaption:SetSize(100,20);
	self.LanguageCaption:SetText(Resource[language][44]..":");

	self.LanguageList=DropDownList();
	self.LanguageList:SetParent(self);
	self.LanguageList:SetPosition(self.LanguageCaption:GetLeft()+self.LanguageCaption:GetWidth()+2,self.LanguageCaption.GetTop()-3);
	self.LanguageList:SetSize(self:GetWidth()-(self.LanguageCaption:GetWidth()+self.LanguageCaption:GetLeft())-65,20);
	self.LanguageList:SetBorderColor(trimColor);
	self.LanguageList:SetBackColor(backColor);
	self.LanguageList.CurrentValue:SetBackColor(backColor);
 	self.LanguageList:SetTextColor(listTextColor);
	self.LanguageList:SetDropRows(3);
	self.LanguageList:SetZOrder(2);

	local tmpLanguage;
	for tmpLanguage in pairs(Resource) do
		self.LanguageList:AddItem(Resource[tmpLanguage][1],tmpLanguage);
	end

	self.LanguageList.SelectedIndexChanged = function ()
		language=self.LanguageList:GetValue();
		self:SetLanguage();
	end

	self.AlertCaption=Turbine.UI.Label();
	self.AlertCaption:SetParent(self);
	self.AlertCaption:SetSize(100,20);
	self.AlertCaption:SetPosition(10,self.LanguageCaption:GetTop()+25);
	self.AlertCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.AlertCaption:SetText(Resource[language][3]..":");
	self.AlertSelect=DropDownList();
	self.AlertSelect:SetParent(self);
	self.AlertSelect:SetSize(180,20);
	self.AlertSelect:SetPosition(self.AlertCaption:GetLeft()+self.AlertCaption:GetWidth()+2,self.AlertCaption:GetTop());
	self.AlertSelect:SetBorderColor(trimColor);
	self.AlertSelect:SetBackColor(backColor);
	self.AlertSelect.CurrentValue:SetBackColor(backColor);
 	self.AlertSelect:SetTextColor(listTextColor);
	self.AlertSelect:SetDropRows(3);
	self.AlertSelect:SetZOrder(1);
	self.AlertSelect.SelectedIndexChanged=function()
		if self.AlertSelect:GetSelectedIndex()>1 then
			local tmpAlertIndex=self.AlertSelect:GetValue();
			local tmpSettingIndex,tmpIndex, tmpFont;
			if Alerts[tmpAlertIndex][1]==nil then
				self.LabelText:SetText("");
			else
				self.LabelText:SetText(Alerts[tmpAlertIndex][1]);
			end
			if Alerts[tmpAlertIndex][2]==nil then
				self.MessageText:SetText("");
			else
				self.MessageText:SetText(Alerts[tmpAlertIndex][2]);
			end
			if Alerts[tmpAlertIndex][3]==nil then
				self.DurationText:SetText("");
			else
				self.DurationText:SetText(Alerts[tmpAlertIndex][3]);
			end
			if Alerts[tmpAlertIndex][4]==nil then
				self.MessageColor:SetColor(FlasherColor);
			else
				self.MessageColor:SetColor(Alerts[tmpAlertIndex][4]);
			end
			tmpSettingIndex=24;
			tmpFont=Alerts[tmpAlertIndex][5];
			if tmpFont==nil then
				tmpFont=FlasherFont;
			end
			for tmpIndex=1,self.FontSelect.ListData:GetItemCount() do
				if self.FontSelect.ListData:GetItem(tmpIndex).DataValue==tmpFont then
					tmpSettingIndex=tmpIndex;
					break;
				end
			end
			self.FontSelect:SetSelectedIndex(tmpSettingIndex);
			if Alerts[tmpAlertIndex][6]==nil then
				self.IntervalText:SetText("");
			else
				self.IntervalText:SetText(Alerts[tmpAlertIndex][6]);
			end
			local m,tmpVal, tmpChannel;
			for tmpIndex=1,#self.ChannelSelect do
				self.ChannelSelect[tmpIndex]:SetChecked(false);
			end
			if Alerts[tmpAlertIndex][7]==nil or type(Alerts[tmpAlertIndex][7])~="table" then
			else
				for tmpIndex=1,#Alerts[tmpAlertIndex][7] do
					for tmpChannel=1,#self.ChannelSelect do
						if Alerts[tmpAlertIndex][7][tmpIndex]==self.ChannelSelect[tmpChannel].Value then
							self.ChannelSelect[tmpChannel]:SetChecked(true);
						end
					end
				end
			end
			self.CustChanText:SetText("");
			if Alerts[tmpAlertIndex][16]~=nil then
				self.CustChanText:SetText(Alerts[tmpAlertIndex][16]);
			end
			if Alerts[tmpAlertIndex][8]==nil then
				self.PatternText:SetText("");
			else
				self.PatternText:SetText(Alerts[tmpAlertIndex][8]);
			end
			if Alerts[tmpAlertIndex][9]==nil then
				self.CooldownText:SetText("");
			else
				self.CooldownText:SetText(Alerts[tmpAlertIndex][9]);
			end
			if Alerts[tmpAlertIndex][11]==nil then
				self.LeftText:SetText("");
				self.LeftPercent:SetText("");
				alertTemplate:SetLeft(0);
			else
				self.LeftText:SetText(Alerts[tmpAlertIndex][11]*Turbine.UI.Display:GetWidth()/100);
				self.LeftPercent:SetText(tostring(math.floor(Alerts[tmpAlertIndex][11]*10+.5)/10).."%");
				alertTemplate:SetLeft(tonumber(self.LeftText:GetText()));
			end
			if Alerts[tmpAlertIndex][12]==nil then
				self.TopText:SetText("");
				self.TopPercent:SetText("");
				alertTemplate:SetTop(0);
			else
				self.TopText:SetText(Alerts[tmpAlertIndex][12]*Turbine.UI.Display:GetHeight()/100);
				self.TopPercent:SetText(tostring(math.floor(Alerts[tmpAlertIndex][12]*10+.5)/10).."%");
				alertTemplate:SetTop(tonumber(self.TopText:GetText()));
			end
			if Alerts[tmpAlertIndex][13]==nil then
				self.WidthText:SetText("");
				self.WidthPercent:SetText("");
				alertTemplate:SetWidth(Turbine.UI.Display:GetWidth()-alertTemplate:GetLeft());
			else
				self.WidthText:SetText(Alerts[tmpAlertIndex][13]*Turbine.UI.Display:GetWidth()/100);
				self.WidthPercent:SetText(tostring(math.floor(Alerts[tmpAlertIndex][13]*10+.5)/10).."%");
				alertTemplate:SetWidth(tonumber(self.WidthText:GetText()));
			end
			if Alerts[tmpAlertIndex][14]==nil then
				self.HeightText:SetText("");
				self.HeightPercent:SetText("");
				alertTemplate:SetHeight(Turbine.UI.Display:GetHeight()-alertTemplate:GetTop());
			else
				self.HeightText:SetText(Alerts[tmpAlertIndex][14]*Turbine.UI.Display:GetHeight()/100);
				self.HeightPercent:SetText(tostring(math.floor(Alerts[tmpAlertIndex][14]*10+.5)/10).."%");
				alertTemplate:SetHeight(tonumber(self.HeightText:GetText()));
			end
			self.ImageText:SetText(Alerts[tmpAlertIndex][15]);
			if Alerts[tmpAlertIndex][17]==nil or Alerts[tmpAlertIndex][17] then
				self.EnabledCB:SetChecked(true);
			else
				self.EnabledCB:SetChecked(false);
			end
		end
	end

	self.AlertSelect:AddItem(Resource[language][4],0);
	for tmpIndex=1,#Alerts do
		self.AlertSelect:AddItem(Alerts[tmpIndex][1],tmpIndex);
	end

	self.LabelCaption=Turbine.UI.Label();
	self.LabelCaption:SetParent(self);
	self.LabelCaption:SetSize(100,20);
	self.LabelCaption:SetPosition(10,self.AlertCaption:GetTop()+25);
	self.LabelCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.LabelCaption:SetText(Resource[language][5]..":");
	self.LabelText=Turbine.UI.Lotro.TextBox();
	self.LabelText:SetParent(self);
	self.LabelText:SetSize(200,20);
	self.LabelText:SetFont(fontFace);
	self.LabelText:SetPosition(self.LabelCaption:GetLeft()+self.LabelCaption:GetWidth()+2,self.LabelCaption:GetTop())

	self.EnabledCB=Turbine.UI.Lotro.CheckBox()
	self.EnabledCB:SetParent(self);
	self.EnabledCB:SetSize(150,20);
	self.EnabledCB:SetPosition(self.LabelText:GetLeft()+self.LabelText:GetWidth()+10,self.LabelCaption:GetTop());
	self.EnabledCB:SetText(Resource[language][59]);
	self.EnabledCB:SetChecked(true);

	self.MessageCaption=Turbine.UI.Label();
	self.MessageCaption:SetParent(self);
	self.MessageCaption:SetSize(100,20);
	self.MessageCaption:SetPosition(10,self.LabelCaption:GetTop()+25);
	self.MessageCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.MessageCaption:SetText(Resource[language][6]..":");
	self.MessageText=Turbine.UI.Lotro.TextBox();
	self.MessageText:SetParent(self);
	self.MessageText:SetSize(self:GetWidth()-self.MessageCaption:GetLeft()-self.MessageCaption:GetWidth()-12,20);
	self.MessageText:SetFont(fontFace);
	self.MessageText:SetPosition(self.MessageCaption:GetLeft()+self.MessageCaption:GetWidth()+2,self.MessageCaption:GetTop());

	self.DurationCaption=Turbine.UI.Label();
	self.DurationCaption:SetParent(self);
	self.DurationCaption:SetSize(100,20);
	self.DurationCaption:SetPosition(10,self.MessageCaption:GetTop()+25);
	self.DurationCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.DurationCaption:SetText(Resource[language][9]..":");
	self.DurationText=Turbine.UI.Lotro.TextBox();
	self.DurationText:SetParent(self);
	self.DurationText:SetSize(80,20);
	self.DurationText:SetFont(fontFace);
	self.DurationText:SetPosition(self.DurationCaption:GetLeft()+self.DurationCaption:GetWidth()+2,self.DurationCaption:GetTop());

	self.IntervalCaption=Turbine.UI.Label();
	self.IntervalCaption:SetParent(self);
	self.IntervalCaption:SetSize(100,20);
	self.IntervalCaption:SetPosition(self:GetWidth()/2,self.MessageCaption:GetTop()+25);
	self.IntervalCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.IntervalCaption:SetText(Resource[language][10]..":");
	self.IntervalText=Turbine.UI.Lotro.TextBox();
	self.IntervalText:SetParent(self);
	self.IntervalText:SetSize(80,20);
	self.IntervalText:SetFont(fontFace);
	self.IntervalText:SetPosition(self.IntervalCaption:GetLeft()+self.IntervalCaption:GetWidth()+2,self.IntervalCaption:GetTop());

	self.CooldownCaption=Turbine.UI.Label();
	self.CooldownCaption:SetParent(self);
	self.CooldownCaption:SetSize(100,20);
	self.CooldownCaption:SetPosition(10,self.DurationCaption:GetTop()+25);
	self.CooldownCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.CooldownCaption:SetText(Resource[language][45]..":");
	self.CooldownText=Turbine.UI.Lotro.TextBox();
	self.CooldownText:SetParent(self);
	self.CooldownText:SetSize(80,20);
	self.CooldownText:SetFont(fontFace);
	self.CooldownText:SetPosition(self.CooldownCaption:GetLeft()+self.CooldownCaption:GetWidth()+2,self.CooldownCaption:GetTop());

	self.LeftCaption=Turbine.UI.Label();
	self.LeftCaption:SetParent(self);
	self.LeftCaption:SetSize(100,20);
	self.LeftCaption:SetPosition(10,self.CooldownCaption:GetTop()+25);
	self.LeftCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.LeftCaption:SetText(Resource[language][46]..":");
	self.LeftText=Turbine.UI.Lotro.TextBox();
	self.LeftText:SetParent(self);
	self.LeftText:SetSize(80,20);
	self.LeftText:SetFont(fontFace);
	self.LeftText:SetPosition(self.LeftCaption:GetLeft()+self.LeftCaption:GetWidth()+2,self.LeftCaption:GetTop());
	self.LeftPercent=Turbine.UI.Label();
	self.LeftPercent:SetParent(self);
	self.LeftPercent:SetSize(40,20);
	self.LeftPercent:SetPosition(self.LeftText:GetLeft()+self.LeftText:GetWidth()+2,self.LeftCaption:GetTop());
	self.LeftPercent:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.LeftPercent:SetText("");

	self.TopCaption=Turbine.UI.Label();
	self.TopCaption:SetParent(self);
	self.TopCaption:SetSize(100,20);
	self.TopCaption:SetPosition(self:GetWidth()/2,self.CooldownCaption:GetTop()+25);
	self.TopCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.TopCaption:SetText(Resource[language][47]..":");
	self.TopText=Turbine.UI.Lotro.TextBox();
	self.TopText:SetParent(self);
	self.TopText:SetSize(80,20);
	self.TopText:SetFont(fontFace);
	self.TopText:SetPosition(self.TopCaption:GetLeft()+self.TopCaption:GetWidth()+2,self.TopCaption:GetTop());
	self.TopPercent=Turbine.UI.Label();
	self.TopPercent:SetParent(self);
	self.TopPercent:SetSize(40,20);
	self.TopPercent:SetPosition(self.TopText:GetLeft()+self.TopText:GetWidth()+2,self.TopCaption:GetTop());
	self.TopPercent:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.TopPercent:SetText("");

	self.WidthCaption=Turbine.UI.Label();
	self.WidthCaption:SetParent(self);
	self.WidthCaption:SetSize(100,20);
	self.WidthCaption:SetPosition(10,self.LeftCaption:GetTop()+25);
	self.WidthCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.WidthCaption:SetText(Resource[language][48]..":");
	self.WidthText=Turbine.UI.Lotro.TextBox();
	self.WidthText:SetParent(self);
	self.WidthText:SetSize(80,20);
	self.WidthText:SetFont(fontFace);
	self.WidthText:SetPosition(self.WidthCaption:GetLeft()+self.WidthCaption:GetWidth()+2,self.WidthCaption:GetTop());
	self.WidthPercent=Turbine.UI.Label();
	self.WidthPercent:SetParent(self);
	self.WidthPercent:SetSize(40,20);
	self.WidthPercent:SetPosition(self.WidthText:GetLeft()+self.WidthText:GetWidth()+2,self.WidthCaption:GetTop());
	self.WidthPercent:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.WidthPercent:SetText("");

	self.HeightCaption=Turbine.UI.Label();
	self.HeightCaption:SetParent(self);
	self.HeightCaption:SetSize(100,20);
	self.HeightCaption:SetPosition(self:GetWidth()/2,self.LeftCaption:GetTop()+25);
	self.HeightCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.HeightCaption:SetText(Resource[language][49]..":");
	self.HeightText=Turbine.UI.Lotro.TextBox();
	self.HeightText:SetParent(self);
	self.HeightText:SetSize(80,20);
	self.HeightText:SetFont(fontFace);
	self.HeightText:SetPosition(self.HeightCaption:GetLeft()+self.HeightCaption:GetWidth()+2,self.HeightCaption:GetTop());
	self.HeightPercent=Turbine.UI.Label();
	self.HeightPercent:SetParent(self);
	self.HeightPercent:SetSize(40,20);
	self.HeightPercent:SetPosition(self.HeightText:GetLeft()+self.HeightText:GetWidth()+2,self.HeightCaption:GetTop());
	self.HeightPercent:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.HeightPercent:SetText("");

	self.TopText.Update=function(sender,args)
		local val=tonumber(self.LeftText:GetText());
		if val==nil then
			self.LeftPercent:SetText("");
			alertTemplate:SetLeft(0);
		else
			self.LeftPercent:SetText(tostring(math.floor(val/Turbine.UI.Display:GetWidth()*1000+.5)/10).."%")
			alertTemplate:SetLeft(val);
		end
		val=tonumber(self.TopText:GetText());
		if val==nil then
			self.TopPercent:SetText("");
			alertTemplate:SetTop(0);
		else
			self.TopPercent:SetText(tostring(math.floor(val/Turbine.UI.Display:GetHeight()*1000+.5)/10).."%")
			alertTemplate:SetTop(val);
		end
		val=tonumber(self.WidthText:GetText());
		if val==nil then
			self.WidthPercent:SetText("");
			alertTemplate:SetWidth(Turbine.UI.Display:GetWidth()-alertTemplate:GetLeft());
		else
			self.WidthPercent:SetText(tostring(math.floor(val/Turbine.UI.Display:GetWidth()*1000+.5)/10).."%")
			alertTemplate:SetWidth(val);
		end
		val=tonumber(self.HeightText:GetText());
		if val==nil then
			self.HeightPercent:SetText("");
			alertTemplate:SetHeight(Turbine.UI.Display:GetHeight()-alertTemplate:GetTop());
		else
			self.HeightPercent:SetText(tostring(math.floor(val/Turbine.UI.Display:GetHeight()*1000+.5)/10).."%")
			alertTemplate:SetHeight(val);
		end
		alertTemplate:SetWidth(self.WidthText:GetText());

	end
	self.TopText.FocusGained=function()
		self.TopText:SetWantsUpdates(true);
	end
	self.TopText.FocusLost=function()
		self.TopText:SetWantsUpdates(false);
	end
	self.LeftText.FocusGained=function()
		self.TopText:SetWantsUpdates(true);
	end
	self.LeftText.FocusLost=function()
		self.TopText:SetWantsUpdates(false);
	end
	self.WidthText.FocusGained=function()
		self.TopText:SetWantsUpdates(true);
	end
	self.WidthText.FocusLost=function()
		self.TopText:SetWantsUpdates(false);
	end
	self.HeightText.FocusGained=function()
		self.TopText:SetWantsUpdates(true);
	end
	self.HeightText.FocusLost=function()
		self.TopText:SetWantsUpdates(false);
	end

	self.ColorCaption=Turbine.UI.Label();
	self.ColorCaption:SetParent(self);
	self.ColorCaption:SetSize(100,20);
	self.ColorCaption:SetPosition(10,self.WidthCaption:GetTop()+25);
	self.ColorCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.ColorCaption:SetText(Resource[language][7]..":");

	self.MessageColor=ColorPicker();
	self.MessageColor:SetParent(self);
	self.MessageColor:SetPosition(self.ColorCaption:GetLeft()+self.ColorCaption:GetWidth()+2,self.ColorCaption:GetTop()-1);
	self.MessageColor:SetTrimColor(Turbine.UI.Color(.4,.4,.4));
	self.MessageColor:SetColor(Turbine.UI.Color(1,0,0));
--	self.MessageColor.ColorChanged=function()
--		self:SetMessageColor(self.MessageColor:GetColor());
--	end

	self.FontCaption=Turbine.UI.Label();
	self.FontCaption:SetParent(self);
	self.FontCaption:SetSize(100,20);
	self.FontCaption:SetPosition(10,self.ColorCaption:GetTop()+25);
	self.FontCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.FontCaption:SetText(Resource[language][8]..":");
	self.FontSelect=DropDownList();
	self.FontSelect:SetParent(self);
	self.FontSelect:SetSize(180,20);
	self.FontSelect:SetPosition(self.FontCaption:GetLeft()+self.FontCaption:GetWidth()+2,self.FontCaption:GetTop());
	self.FontSelect:SetBorderColor(trimColor);
	self.FontSelect:SetBackColor(backColor);
	self.FontSelect.CurrentValue:SetBackColor(backColor);
 	self.FontSelect:SetTextColor(listTextColor);
	self.FontSelect:SetDropRows(3);
	self.FontSelect:SetZOrder(1);
	self.FontSelect.SelectedIndexChanged=function()
	end
	self.FontSelect:AddItem("Arial 12",Turbine.UI.Lotro.Font.Arial12);
	self.FontSelect:AddItem("Calibri 14",0x4200002b);
	self.FontSelect:AddItem("TimesRoman 12",0x42000021);
	self.FontSelect:AddItem("TimesRoman 20",0x4200000e);
	self.FontSelect:AddItem("TimesRoman 22",0x4200000f);
	self.FontSelect:AddItem("TimesRoman 24",0x42000010);
	self.FontSelect:AddItem("TrajanPro 13",Turbine.UI.Lotro.Font.TrajanPro13);
	self.FontSelect:AddItem("TrajanPro 14",Turbine.UI.Lotro.Font.TrajanPro14);
	self.FontSelect:AddItem("TrajanPro 15",Turbine.UI.Lotro.Font.TrajanPro15);
	self.FontSelect:AddItem("TrajanPro 16",Turbine.UI.Lotro.Font.TrajanPro16);
	self.FontSelect:AddItem("TrajanPro 18",Turbine.UI.Lotro.Font.TrajanPro18);
	self.FontSelect:AddItem("TrajanPro 19",Turbine.UI.Lotro.Font.TrajanPro19);
	self.FontSelect:AddItem("TrajanPro 20",Turbine.UI.Lotro.Font.TrajanPro20);
	self.FontSelect:AddItem("TrajanPro 21",Turbine.UI.Lotro.Font.TrajanPro21);
	self.FontSelect:AddItem("TrajanPro 23",Turbine.UI.Lotro.Font.TrajanPro23);
	self.FontSelect:AddItem("TrajanPro 24",Turbine.UI.Lotro.Font.TrajanPro24);
--	self.FontSelect:AddItem("TrajanPro 25",Turbine.UI.Lotro.Font.TrajanPro25);
	self.FontSelect:AddItem("TrajanPro 26",Turbine.UI.Lotro.Font.TrajanPro26);
	self.FontSelect:AddItem("TrajanPro 28",Turbine.UI.Lotro.Font.TrajanPro28);
	self.FontSelect:AddItem("TrajanProBold 16",Turbine.UI.Lotro.Font.TrajanProBold16);
	self.FontSelect:AddItem("TrajanProBold 22",Turbine.UI.Lotro.Font.TrajanProBold22);
	self.FontSelect:AddItem("TrajanProBold 24",Turbine.UI.Lotro.Font.TrajanProBold24);
	self.FontSelect:AddItem("TrajanProBold 25",Turbine.UI.Lotro.Font.TrajanProBold25);
	self.FontSelect:AddItem("TrajanProBold 30",Turbine.UI.Lotro.Font.TrajanProBold30);
	self.FontSelect:AddItem("TrajanProBold 36",Turbine.UI.Lotro.Font.TrajanProBold36);
	self.FontSelect:AddItem("Verdana 10",Turbine.UI.Lotro.Font.Verdana10);
	self.FontSelect:AddItem("Verdana 12",Turbine.UI.Lotro.Font.Verdana12);
	self.FontSelect:AddItem("Verdana 14",Turbine.UI.Lotro.Font.Verdana14);
	self.FontSelect:AddItem("Verdana 16",Turbine.UI.Lotro.Font.Verdana16);
	self.FontSelect:AddItem("Verdana 18",Turbine.UI.Lotro.Font.Verdana18);
	self.FontSelect:AddItem("Verdana 20",Turbine.UI.Lotro.Font.Verdana20);
	self.FontSelect:AddItem("Verdana 22",Turbine.UI.Lotro.Font.Verdana22);
	self.FontSelect:AddItem("Verdana 23",Turbine.UI.Lotro.Font.Verdana23);

	self.FontSelect:SetSelectedIndex(24); -- default to the largest

	self.ImageCaption=Turbine.UI.Label();
	self.ImageCaption:SetParent(self);
	self.ImageCaption:SetSize(100,20);
	self.ImageCaption:SetPosition(10,self.FontCaption:GetTop()+25);
	self.ImageCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.ImageCaption:SetText(Resource[language][50]..":");
	self.ImageText=Turbine.UI.Lotro.TextBox();
	self.ImageText:SetParent(self);
	self.ImageText:SetSize(self:GetWidth()-self.ImageCaption:GetLeft()-self.ImageCaption:GetWidth()-12,20);
	self.ImageText:SetFont(fontFace);
	self.ImageText:SetPosition(self.ImageCaption:GetLeft()+self.ImageCaption:GetWidth()+2,self.ImageCaption:GetTop());

	self.ChannelCaption=Turbine.UI.Label();
	self.ChannelCaption:SetParent(self);
	self.ChannelCaption:SetSize(self:GetWidth()-20,20);
	self.ChannelCaption:SetPosition(10,self.ImageCaption:GetTop()+25);
	self.ChannelCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.ChannelCaption:SetText(Resource[language][12]..":");

	local offset=(self:GetWidth()-20)/5;
	self.ChannelSelect={}
	self.ChannelSelect[1]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[1]:SetParent(self)
	self.ChannelSelect[1]:SetText(Resource[language][14]);
	self.ChannelSelect[1]:SetSize(offset-10,20);
	self.ChannelSelect[1]:SetPosition(10,self.ChannelCaption:GetTop()+25);
	self.ChannelSelect[1].Value=0;
	self.ChannelSelect[2]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[2]:SetParent(self)
	self.ChannelSelect[2]:SetText(Resource[language][15]);
	self.ChannelSelect[2]:SetSize(offset-10,20);
	self.ChannelSelect[2]:SetPosition(offset,self.ChannelCaption:GetTop()+25);
	self.ChannelSelect[2].Value=Turbine.ChatType.Admin
	self.ChannelSelect[3]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[3]:SetParent(self)
	self.ChannelSelect[3]:SetText(Resource[language][16]);
	self.ChannelSelect[3]:SetSize(offset-10,20);
	self.ChannelSelect[3]:SetPosition(offset*2,self.ChannelCaption:GetTop()+25);
	self.ChannelSelect[3].Value=Turbine.ChatType.Advancement
	self.ChannelSelect[4]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[4]:SetParent(self)
	self.ChannelSelect[4]:SetText(Resource[language][17]);
	self.ChannelSelect[4]:SetSize(offset-10,20);
	self.ChannelSelect[4]:SetPosition(offset*3,self.ChannelCaption:GetTop()+25);
	self.ChannelSelect[4].Value=Turbine.ChatType.Advice
	self.ChannelSelect[5]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[5]:SetParent(self)
	self.ChannelSelect[5]:SetText(Resource[language][18]);
	self.ChannelSelect[5]:SetSize(offset-10,20);
	self.ChannelSelect[5]:SetPosition(offset*4,self.ChannelCaption:GetTop()+25);
	self.ChannelSelect[5].Value=Turbine.ChatType.Combat
	self.ChannelSelect[6]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[6]:SetParent(self)
	self.ChannelSelect[6]:SetText(Resource[language][19]);
	self.ChannelSelect[6]:SetSize(offset-10,20);
	self.ChannelSelect[6]:SetPosition(10,self.ChannelCaption:GetTop()+50);
	self.ChannelSelect[6].Value=Turbine.ChatType.Death
	self.ChannelSelect[7]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[7]:SetParent(self)
	self.ChannelSelect[7]:SetText(Resource[language][20]);
	self.ChannelSelect[7]:SetSize(offset-10,20);
	self.ChannelSelect[7]:SetPosition(offset,self.ChannelCaption:GetTop()+50);
	self.ChannelSelect[7].Value=Turbine.ChatType.Emote
	self.ChannelSelect[8]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[8]:SetParent(self)
	self.ChannelSelect[8]:SetText(Resource[language][21]);
	self.ChannelSelect[8]:SetSize(offset-10,20);
	self.ChannelSelect[8]:SetPosition(offset*2,self.ChannelCaption:GetTop()+50);
	self.ChannelSelect[8].Value=Turbine.ChatType.Error
	self.ChannelSelect[9]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[9]:SetParent(self)
	self.ChannelSelect[9]:SetText(Resource[language][22]);
	self.ChannelSelect[9]:SetSize(offset-10,20);
	self.ChannelSelect[9]:SetPosition(offset*3,self.ChannelCaption:GetTop()+50);
	self.ChannelSelect[9].Value=Turbine.ChatType.Fellowship
	self.ChannelSelect[10]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[10]:SetParent(self)
	self.ChannelSelect[10]:SetText(Resource[language][23]);
	self.ChannelSelect[10]:SetSize(offset-10,20);
	self.ChannelSelect[10]:SetPosition(offset*4,self.ChannelCaption:GetTop()+50);
	self.ChannelSelect[10].Value=Turbine.ChatType.Kinship
	self.ChannelSelect[11]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[11]:SetParent(self)
	self.ChannelSelect[11]:SetText(Resource[language][24]);
	self.ChannelSelect[11]:SetSize(offset-10,20);
	self.ChannelSelect[11]:SetPosition(10,self.ChannelCaption:GetTop()+75);
	self.ChannelSelect[11].Value=Turbine.ChatType.LFF
	self.ChannelSelect[12]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[12]:SetParent(self)
	self.ChannelSelect[12]:SetText(Resource[language][25]);
	self.ChannelSelect[12]:SetSize(offset-10,20);
	self.ChannelSelect[12]:SetPosition(offset,self.ChannelCaption:GetTop()+75);
	self.ChannelSelect[12].Value=Turbine.ChatType.Officer
	self.ChannelSelect[13]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[13]:SetParent(self)
	self.ChannelSelect[13]:SetText(Resource[language][26]);
	self.ChannelSelect[13]:SetSize(offset-10,20);
	self.ChannelSelect[13]:SetPosition(offset*2,self.ChannelCaption:GetTop()+75);
	self.ChannelSelect[13].Value=Turbine.ChatType.OOC
	self.ChannelSelect[14]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[14]:SetParent(self)
	self.ChannelSelect[14]:SetText(Resource[language][27]);
	self.ChannelSelect[14]:SetSize(offset-10,20);
	self.ChannelSelect[14]:SetPosition(offset*3,self.ChannelCaption:GetTop()+75);
	self.ChannelSelect[14].Value=Turbine.ChatType.Quest
	self.ChannelSelect[15]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[15]:SetParent(self)
	self.ChannelSelect[15]:SetText(Resource[language][28]);
	self.ChannelSelect[15]:SetSize(offset-10,20);
	self.ChannelSelect[15]:SetPosition(offset*4,self.ChannelCaption:GetTop()+75);
	self.ChannelSelect[15].Value=Turbine.ChatType.Raid
	self.ChannelSelect[16]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[16]:SetParent(self)
	self.ChannelSelect[16]:SetText(Resource[language][29]);
	self.ChannelSelect[16]:SetSize(offset-10,20);
	self.ChannelSelect[16]:SetPosition(10,self.ChannelCaption:GetTop()+100);
	self.ChannelSelect[16].Value=Turbine.ChatType.Regional
	self.ChannelSelect[17]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[17]:SetParent(self)
	self.ChannelSelect[17]:SetText(Resource[language][30]);
	self.ChannelSelect[17]:SetSize(offset-10,20);
	self.ChannelSelect[17]:SetPosition(offset,self.ChannelCaption:GetTop()+100);
	self.ChannelSelect[17].Value=Turbine.ChatType.Roleplay
	self.ChannelSelect[18]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[18]:SetParent(self)
	self.ChannelSelect[18]:SetText(Resource[language][31]);
	self.ChannelSelect[18]:SetSize(offset-10,20);
	self.ChannelSelect[18]:SetPosition(offset*2,self.ChannelCaption:GetTop()+100);
	self.ChannelSelect[18].Value=Turbine.ChatType.Say
	self.ChannelSelect[19]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[19]:SetParent(self)
	self.ChannelSelect[19]:SetText(Resource[language][32]);
	self.ChannelSelect[19]:SetSize(offset-10,20);
	self.ChannelSelect[19]:SetPosition(offset*3,self.ChannelCaption:GetTop()+100);
	self.ChannelSelect[19].Value=Turbine.ChatType.Standard
	self.ChannelSelect[20]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[20]:SetParent(self)
	self.ChannelSelect[20]:SetText(Resource[language][33]);
	self.ChannelSelect[20]:SetSize(offset-10,20);
	self.ChannelSelect[20]:SetPosition(offset*4,self.ChannelCaption:GetTop()+100);
	self.ChannelSelect[20].Value=Turbine.ChatType.Tell
	self.ChannelSelect[21]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[21]:SetParent(self)
	self.ChannelSelect[21]:SetText(Resource[language][34]);
	self.ChannelSelect[21]:SetSize(offset-10,20);
	self.ChannelSelect[21]:SetPosition(10,self.ChannelCaption:GetTop()+125);
	self.ChannelSelect[21].Value=Turbine.ChatType.Trade
	self.ChannelSelect[22]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[22]:SetParent(self)
	self.ChannelSelect[22]:SetText(Resource[language][35]);
	self.ChannelSelect[22]:SetSize(offset-10,20);
	self.ChannelSelect[22]:SetPosition(offset,self.ChannelCaption:GetTop()+125);
	self.ChannelSelect[22].Value=Turbine.ChatType.Tribe
	self.ChannelSelect[23]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[23]:SetParent(self)
	self.ChannelSelect[23]:SetText(Resource[language][36]);
	self.ChannelSelect[23]:SetSize(offset-10,20);
	self.ChannelSelect[23]:SetPosition(offset*2,self.ChannelCaption:GetTop()+125);
	self.ChannelSelect[23].Value=Turbine.ChatType.Unfiltered
	self.ChannelSelect[24]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[24]:SetParent(self)
	self.ChannelSelect[24]:SetText(Resource[language][37]);
	self.ChannelSelect[24]:SetSize(offset-10,20);
	self.ChannelSelect[24]:SetPosition(offset*3,self.ChannelCaption:GetTop()+125);
	self.ChannelSelect[24].Value=Turbine.ChatType.UserChat1
	self.ChannelSelect[25]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[25]:SetParent(self)
	self.ChannelSelect[25]:SetText(Resource[language][38]);
	self.ChannelSelect[25]:SetSize(offset-10,20);
	self.ChannelSelect[25]:SetPosition(offset*4,self.ChannelCaption:GetTop()+125);
	self.ChannelSelect[25].Value=Turbine.ChatType.UserChat2
	self.ChannelSelect[26]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[26]:SetParent(self)
	self.ChannelSelect[26]:SetText(Resource[language][39]);
	self.ChannelSelect[26]:SetSize(offset-10,20);
	self.ChannelSelect[26]:SetPosition(10,self.ChannelCaption:GetTop()+150);
	self.ChannelSelect[26].Value=Turbine.ChatType.UserChat3
	self.ChannelSelect[27]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[27]:SetParent(self)
	self.ChannelSelect[27]:SetText(Resource[language][40]);
	self.ChannelSelect[27]:SetSize(offset-10,20);
	self.ChannelSelect[27]:SetPosition(offset,self.ChannelCaption:GetTop()+150);
	self.ChannelSelect[27].Value=Turbine.ChatType.UserChat4
	self.ChannelSelect[28]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[28]:SetParent(self)
	self.ChannelSelect[28]:SetText(Resource[language][54]);
	self.ChannelSelect[28]:SetSize(offset-10,20);
	self.ChannelSelect[28]:SetPosition(offset*2,self.ChannelCaption:GetTop()+150);
	self.ChannelSelect[28].Value=16777232 -- NPC spew
	self.ChannelSelect[29]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[29]:SetParent(self)
	self.ChannelSelect[29]:SetText(Resource[language][55]);
	self.ChannelSelect[29]:SetSize(offset-10,20);
	self.ChannelSelect[29]:SetPosition(offset*3,self.ChannelCaption:GetTop()+150);
	self.ChannelSelect[29].Value=16777280 -- Emotes without text

	self.CustChanCaption=Turbine.UI.Label();
	self.CustChanCaption:SetParent(self);
	self.CustChanCaption:SetSize(120,20);
	self.CustChanCaption:SetPosition(10,self.ChannelCaption:GetTop()+175);
	self.CustChanCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.CustChanCaption:SetText(Resource[language][56]..":");
	self.CustChanText=Turbine.UI.Lotro.TextBox();
	self.CustChanText:SetParent(self);
	self.CustChanText:SetSize(100,20);
	self.CustChanText:SetFont(fontFace);
	self.CustChanText:SetPosition(self.CustChanCaption:GetLeft()+self.CustChanCaption:GetWidth()+2,self.CustChanCaption:GetTop());

	self.PatternCaption=Turbine.UI.Label();
	self.PatternCaption:SetParent(self);
	self.PatternCaption:SetSize(100,20);
	self.PatternCaption:SetPosition(10,self.CustChanCaption:GetTop()+25);
	self.PatternCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.PatternCaption:SetText(Resource[language][13]..":");
	self.PatternText=Turbine.UI.Lotro.TextBox();
	self.PatternText:SetParent(self);
	self.PatternText:SetSize(self:GetWidth()-self.PatternCaption:GetLeft()-self.PatternCaption:GetWidth()-12,20);
	self.PatternText:SetFont(fontFace);
	self.PatternText:SetPosition(self.PatternCaption:GetLeft()+self.PatternCaption:GetWidth()+2,self.PatternCaption:GetTop());

	self.SaveButton=Turbine.UI.Lotro.Button();
	self.SaveButton:SetParent(self);
	self.SaveButton:SetSize(100,20);
	self.SaveButton:SetText(Resource[language][41]);
	self.SaveButton:SetPosition(20,self:GetHeight()-45);
	self.SaveButton.MouseClick=function()
		-- if the alertselect is on New, add the definition to the Alerts, otherwise update the one pointed to by the select list
		local tmpIndex;
		local tmpAlertIndex=self.AlertSelect:GetValue();
		if tmpAlertIndex==0 then
			tmpAlertIndex=#Alerts+1
			-- need to add to the alerts list alphabetically
			self.AlertSelect:AddItem(self.LabelText:GetText(),tmpAlertIndex);
			Alerts[tmpAlertIndex]={};
		else
			-- update the alertSelect label
			self.AlertSelect.CurrentValue:SetText(self.LabelText:GetText());
			self.AlertSelect.ListData:GetItem(self.AlertSelect:GetSelectedIndex()):SetText(self.LabelText:GetText());
		end
		Alerts[tmpAlertIndex][1]=self.LabelText:GetText();
		Alerts[tmpAlertIndex][2]=self.MessageText:GetText();
		Alerts[tmpAlertIndex][3]=tonumber(self.DurationText:GetText());
		Alerts[tmpAlertIndex][4]=self.MessageColor:GetColor();
		Alerts[tmpAlertIndex][5]=self.FontSelect:GetValue();
		Alerts[tmpAlertIndex][6]=tonumber(self.IntervalText:GetText());
		Alerts[tmpAlertIndex][7]={};
		for tmpIndex=1,#self.ChannelSelect do
			if self.ChannelSelect[tmpIndex]:IsChecked() then
				Alerts[tmpAlertIndex][7][#Alerts[tmpAlertIndex][7]+1]=self.ChannelSelect[tmpIndex].Value;
			end
		end
		Alerts[tmpAlertIndex][8]=self.PatternText:GetText();
		Alerts[tmpAlertIndex][9]=tonumber(self.CooldownText:GetText());
-- convert to percents
		local percent=tonumber(self.LeftText:GetText())
		if percent==nil then percent=0 end
		Alerts[tmpAlertIndex][11]=percent/Turbine.UI.Display:GetWidth()*100;
		percent=tonumber(self.TopText:GetText())
		if percent==nil then percent=0 end
		Alerts[tmpAlertIndex][12]=percent/Turbine.UI.Display:GetHeight()*100;

		percent=tonumber(self.WidthText:GetText())
		if percent==nil then
			Alerts[tmpAlertIndex][13]=100;
		else
			Alerts[tmpAlertIndex][13]=percent/Turbine.UI.Display:GetWidth()*100;
		end
		percent=tonumber(self.HeightText:GetText())
		if percent==nil then
			Alerts[tmpAlertIndex][14]=100;
		else
			Alerts[tmpAlertIndex][14]=percent/Turbine.UI.Display:GetHeight()*100;
		end

		Alerts[tmpAlertIndex][15]=string.trim(self.ImageText:GetText());
		if self.AlertSelect:GetSelectedIndex()==1 then
			self.AlertSelect:SetSelectedIndex(self.AlertSelect.ListData:GetItemCount());
		end
		Alerts[tmpAlertIndex][16]=tonumber(self.CustChanText:GetText());
		Alerts[tmpAlertIndex][17]=self.EnabledCB:IsChecked();
	end

	self.TestButton=Turbine.UI.Lotro.Button();
	self.TestButton:SetParent(self);
	self.TestButton:SetSize(100,20);
	self.TestButton:SetText(Resource[language][42]);
	self.TestButton:SetPosition((self:GetWidth()-40-self.TestButton:GetWidth()*4)/3+self.TestButton:GetWidth()+20,self:GetHeight()-45);
	self.TestButton.MouseClick=function()

		local leftPct=tonumber(self.LeftText:GetText())
		if leftPct==nil then leftPct=0 end
		leftPct=leftPct/Turbine.UI.Display:GetWidth()*100;

		local topPct=tonumber(self.TopText:GetText())
		if topPct==nil then topPct=0 end
		topPct=topPct/Turbine.UI.Display:GetHeight()*100;

		local widthPct=tonumber(self.WidthText:GetText())
		if widthPct==nil then
			widthPct=100;
		else
			widthPct=widthPct/Turbine.UI.Display:GetWidth()*100;
		end
		local heightPct=tonumber(self.HeightText:GetText())
		if heightPct==nil then
			heightPct=100;
		else
			heightPct=heightPct/Turbine.UI.Display:GetHeight()*100;
		end
		if self.MessageText:GetText()=="" then
			self.Flasher.SetAlert(self.PatternText:GetText(), tonumber(self.DurationText:GetText()), self.MessageColor:GetColor(), self.FontSelect:GetValue(), tonumber(self.IntervalText:GetText()), tonumber(leftPct), tonumber(topPct), tonumber(widthPct), tonumber(heightPct), self.ImageText:GetText());
		else
			self.Flasher.SetAlert(self.MessageText:GetText(), tonumber(self.DurationText:GetText()), self.MessageColor:GetColor(), self.FontSelect:GetValue(), tonumber(self.IntervalText:GetText()), tonumber(leftPct), tonumber(topPct), tonumber(widthPct), tonumber(heightPct), self.ImageText:GetText());
		end
	end

	self.LogButton=Turbine.UI.Lotro.Button();
	self.LogButton:SetParent(self);
	self.LogButton:SetSize(100,20);
	self.LogButton:SetText(Resource[language][53]);
	self.LogButton:SetPosition((self:GetWidth()-40-self.LogButton:GetWidth()*4)*2/3+self.LogButton:GetWidth()*2+20,self:GetHeight()-45);
	self.LogButton.MouseClick=function()
		alertLog:SetVisible(true);
	end

	self.DeleteButton=Turbine.UI.Lotro.Button();
	self.DeleteButton:SetParent(self);
	self.DeleteButton:SetSize(100,20);
	self.DeleteButton:SetText(Resource[language][43]);
	self.DeleteButton:SetPosition(self:GetWidth()-self.DeleteButton:GetWidth()-20,self:GetHeight()-45);
	self.DeleteButton.MouseClick=function()
		local tmpListIndex=self.AlertSelect:GetSelectedIndex();
		if tmpListIndex>1 then
			local tmpAlertIndex=self.AlertSelect:GetValue();
			local tmpIndex, tmpLabel;
			tmpLabel=Alerts[tmpAlertIndex][2];
			table.remove(Alerts,tmpAlertIndex);
			for tmpIndex=1,self.AlertSelect.ListData:GetItemCount() do
				if self.AlertSelect.ListData:GetItem(tmpIndex).DataValue>tmpAlertIndex then
					self.AlertSelect.ListData:GetItem(tmpIndex).DataValue=self.AlertSelect.ListData:GetItem(tmpIndex).DataValue-1;
				end
			end
			self.AlertSelect:RemoveItemAt(tmpListIndex);
			self.AlertSelect:SetSelectedIndex(1);
			if tmpListIndex>self.AlertSelect.ListData:GetItemCount() then
				self.AlertSelect:SetSelectedIndex(self.AlertSelect.ListData:GetItemCount());
			else
				self.AlertSelect:SetSelectedIndex(tmpListIndex);
			end
			-- flash deleted message
			alertMain.Flasher.SetAlert("Alert: "..tmpLabel.." DELETED", 1, Turbine.UI.Color(1,1,1), nil, 0);
		end
	end

	self.KeyDown=function(sender, args)
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			self:SetVisible(false);
			alertTemplate:SetVisible(false);
			alertLog:SetVisible(false);
		end
		if ( args.Action == 268435635 ) then -- toggle HUD
			alertHUDState = not alertHUDState;
			if alertHUDState then
				alertMain:SetVisible(alertMainState);
				alertTemplate:SetVisible(alertMainState);
				alertLog:SetVisible(alertLogState);
			else
				alertMainState=alertMain:IsVisible();
				alertLogState=alertLog:IsVisible();
				alertMain:SetVisible(false);
				alertTemplate:SetVisible(false);
				alertLog:SetVisible(false);
			end
		end
	end
	self.loaded=false;
	self:SetWantsUpdates(true);
	self.Update=function()
		if not self.loaded then
			self.loaded=true;
			Plugins["Alerter"].Unload = function(self,sender,args)
				UnloadPlugin();
			end
			self:SetWantsUpdates(false);
			Turbine.Shell.WriteLine("Alerter "..Plugins["Alerter"]:GetVersion().." by Garan loaded");
		end
	end

	self.RemoveShellCommands=function(sender,args)
		Turbine.Shell.RemoveCommand(self.shellCmd)
	end

	self.shellCmd=Turbine.ShellCommand();
	local numberOfCommandsRegistered = Turbine.Shell.AddCommand("Alerter",self.shellCmd);

	self.shellCmd.Execute = function(sender, cmd, args)
		if (string.lower(cmd) == "alerter") then
			local pos1=string.find(args," ",0,true);
			local command;
			if pos1~=nil and pos1>0 then
				command=string.lower(string.sub(args,1,pos1-1));
			else
				command=string.lower(args);
			end
			if (command=="setup") then
				self:SetVisible(true);
				alertTemplate:SetVisible(true);
			elseif (string.lower(args)=="log start") then
				logChat=true;
				alertLog.StartButton:SetText(Resource[language][58]);
			elseif (string.lower(args)=="log stop") then
				logChat=false;
				alertLog.StartButton:SetText(Resource[language][57]);
			elseif (string.lower(args)=="log show") then
				alertLog:SetVisible(true);
			else
				Turbine.Shell.WriteLine("usage:");
				Turbine.Shell.WriteLine("  /Alerter setup");
				Turbine.Shell.WriteLine("       Displays the Alerter Setup window.");
				Turbine.Shell.WriteLine("  /Alerter Log (start|stop|show)");
				Turbine.Shell.WriteLine("       Starts, Stops or Displays the Alerter Chat Log window.");
			end
		end
	end
	self.shellCmd.GetHelp = function(sender, cmd)
		return("usage:\n  /Alerter setup\n       Displays the Alerter Setup window.");
	end

	self.shellCmd.GetShortHelp = function(sender, cmd)
		return("Alerter - usage: /Alerter setup");
	end
	self:SetWantsKeyEvents(true);
	self.VisibleChanged=function()
		if not self:IsVisible() then
			alertTemplate:SetVisible(false);
		end
	end

	self.alerterReceived=function(sender, args)
		local channelMatched;
		local tmpAlert, tmpChannel;
		if logChat then
			-- add to log
			if alertLog.History:GetItemCount()>1000 then
				alertLog.History:RemoveItemAt(1);
			end
			local tmpRow=Turbine.UI.Control();
			tmpRow:SetParent(alertLog.History);
			tmpRow:SetHeight(14);
			tmpRow:SetWidth(alertLog.History:GetWidth());
			tmpRow.Channel=Turbine.UI.Label();
			tmpRow.Channel:SetParent(tmpRow);
			tmpRow.Channel:SetFont(Turbine.UI.Lotro.Font.Verdana14)
			tmpRow.Channel:SetTextAlignment(Turbine.UI.ContentAlignment.TopLeft)
			tmpRow.Channel:SetText(args.ChatType);
			for tmpIndex=1,#self.ChannelSelect do
				if self.ChannelSelect[tmpIndex].Value==args.ChatType then
					tmpRow.Channel:SetText(self.ChannelSelect[tmpIndex]:GetText());
					break;
				end
			end
			tmpRow.Channel:SetSize(80,14);
			tmpRow.Channel:SetPosition(0,0);
			tmpRow.Sender=Turbine.UI.Label();
			tmpRow.Sender:SetParent(tmpRow);
			tmpRow.Sender:SetSize(100,14);
			tmpRow.Sender:SetPosition(80,0);
			tmpRow.Sender:SetFont(Turbine.UI.Lotro.Font.Verdana14)
			tmpRow.Sender:SetTextAlignment(Turbine.UI.ContentAlignment.TopLeft)
			tmpRow.Sender:SetText(args.Sender);
			tmpRow.Msg=Turbine.UI.Label();
			tmpRow.Msg:SetParent(tmpRow);
			tmpRow.Msg:SetSize(tmpRow:GetWidth()-180,14);
			alertLog.tmpText:SetSize(tmpRow:GetWidth()-180,14);
			tmpRow.Msg:SetPosition(180,0);
			tmpRow.Msg:SetFont(Turbine.UI.Lotro.Font.Verdana14)
			tmpRow.Msg:SetTextAlignment(Turbine.UI.ContentAlignment.TopLeft)
			-- there's probably a better way to do this, but this substitution prevents the label from processing the xml tags
			tmpRow.Msg:SetText(string.gsub(args.Message,"<","<>|"));
			alertLog.tmpText:SetText(string.gsub(args.Message,"<","<>|"))
			local tmpPos=string.find(tmpRow.Msg:GetText(),"<>|")
			while tmpPos~=nil do
				tmpRow.Msg:SetSelection(tmpPos,3);
				alertLog.tmpText:SetSelection(tmpPos,3);
				tmpRow.Msg:SetSelectedText("<");
				alertLog.tmpText:SetSelectedText("<");
				tmpPos=string.find(tmpRow.Msg:GetText(),"<>|");
			end
			tmpRow.Msg:SetSelection(0,0);
			tmpRow.Msg:SetSelectable(true);
			alertLog.tmpText:SetHeight(14);
			-- allow the height to autoexpand to 10 rows
			while alertLog.tmpVScroll:IsVisible() and alertLog.tmpText:GetHeight()<140 do
				alertLog.tmpText:SetHeight(alertLog.tmpText:GetHeight()+14);
			end
			tmpRow.Msg:SetHeight(alertLog.tmpText:GetHeight());
			tmpRow:SetHeight(alertLog.tmpText:GetHeight()+1);

			local autoscroll=(alertLog.History:GetItemCount()==0) or (alertLog.History:GetSelectedIndex()==alertLog.History:GetItemCount())

			alertLog.History:AddItem(tmpRow);
			if autoscroll then
				alertLog.History:SetVerticalScrollBar(nil);
				alertLog.History:SetSelectedIndex(alertLog.History:GetItemCount());
				alertLog.History:SetVerticalScrollBar(alertLog.VScroll);
			end
		end

		if args.Message~=nil then
			for tmpIndex=1,#Alerts do
				if (Alerts[tmpIndex][10]==nil or Turbine.Engine:GetGameTime()>=Alerts[tmpIndex][10]) and (Alerts[tmpIndex][17]==nil or Alerts[tmpIndex][17]) then
					channelMatched=false;
					if Alerts[tmpIndex][7]~=nil and type(Alerts[tmpIndex][7])=="table" then
						if Alerts[tmpIndex][16]~=nil and Alerts[tmpIndex][16]==args.ChatType then
							channelMatched=true;
						else
							for tmpAlert=1,#Alerts[tmpIndex][7] do
								if Alerts[tmpIndex][7][tmpAlert]~=nil then
									if Alerts[tmpIndex][7][tmpAlert]==0 or Alerts[tmpIndex][7][tmpAlert]==args.ChatType then
										channelMatched=true;
										break;
									end
								end
							end
						end
					end
					-- channel matched
-- compare to pattern
					if channelMatched then
						local captures=({string.match(string.lower(args.Message),string.lower(Alerts[tmpIndex][8]))});
						if Alerts[tmpIndex][8]==nil or #captures>0 then
							local msg;
--							Turbine.Shell.WriteLine("Channel:["..args.ChatType.."] Sender:"..args.Sender.." Message:"..args.Message)
							if Alerts[tmpIndex][2]==nil or Alerts[tmpIndex][2]=="" then
								msg=args.Message
							else
								msg=Alerts[tmpIndex][2]
							end
							if Alerts[tmpIndex][9]~=nil and Alerts[tmpIndex][9]>0 then
								Alerts[tmpIndex][10]=Turbine.Engine:GetGameTime()+Alerts[tmpIndex][9];
							end
							alertMain.Flasher.SetAlert(msg, Alerts[tmpIndex][3], Alerts[tmpIndex][4], Alerts[tmpIndex][5], Alerts[tmpIndex][6], Alerts[tmpIndex][11], Alerts[tmpIndex][12], Alerts[tmpIndex][13], Alerts[tmpIndex][14], Alerts[tmpIndex][15],captures,args.ChatType,args.Sender,args.Message);
						end
					end
				end
			end
		end
		-- args.ChatType
		-- args.Sender
		-- args.Message
	end
	AddCallback(Turbine.Chat, "Received", self.alerterReceived)

	self.SetLanguage=function()
		self:SetText(Resource[language][2])
		self.AlertCaption:SetText(Resource[language][3]..":");
		self.AlertSelect:AddItem(Resource[language][4],0);
		self.LabelCaption:SetText(Resource[language][5]..":");
		self.MessageCaption:SetText(Resource[language][6]..":");
		self.ColorCaption:SetText(Resource[language][7]..":");
		self.FontCaption:SetText(Resource[language][8]..":");
		self.DurationCaption:SetText(Resource[language][9]..":");
		self.IntervalCaption:SetText(Resource[language][10]..":");
		self.ChannelCaption:SetText(Resource[language][12]..":");
		self.PatternCaption:SetText(Resource[language][13]..":");
		self.LeftCaption:SetText(Resource[language][46]..":");
		self.TopCaption:SetText(Resource[language][47]..":");
		self.WidthCaption:SetText(Resource[language][48]..":");
		self.HeightCaption:SetText(Resource[language][49]..":");
		self.ChannelSelect[1]:SetText(Resource[language][14]);
		self.ChannelSelect[2]:SetText(Resource[language][15]);
		self.ChannelSelect[3]:SetText(Resource[language][16]);
		self.ChannelSelect[4]:SetText(Resource[language][17]);
		self.ChannelSelect[5]:SetText(Resource[language][18]);
		self.ChannelSelect[6]:SetText(Resource[language][19]);
		self.ChannelSelect[7]:SetText(Resource[language][20]);
		self.ChannelSelect[8]:SetText(Resource[language][21]);
		self.ChannelSelect[9]:SetText(Resource[language][22]);
		self.ChannelSelect[10]:SetText(Resource[language][23]);
		self.ChannelSelect[11]:SetText(Resource[language][24]);
		self.ChannelSelect[12]:SetText(Resource[language][25]);
		self.ChannelSelect[13]:SetText(Resource[language][26]);
		self.ChannelSelect[14]:SetText(Resource[language][27]);
		self.ChannelSelect[15]:SetText(Resource[language][28]);
		self.ChannelSelect[16]:SetText(Resource[language][29]);
		self.ChannelSelect[17]:SetText(Resource[language][30]);
		self.ChannelSelect[18]:SetText(Resource[language][31]);
		self.ChannelSelect[19]:SetText(Resource[language][32]);
		self.ChannelSelect[20]:SetText(Resource[language][33]);
		self.ChannelSelect[21]:SetText(Resource[language][34]);
		self.ChannelSelect[22]:SetText(Resource[language][35]);
		self.ChannelSelect[23]:SetText(Resource[language][36]);
		self.ChannelSelect[24]:SetText(Resource[language][37]);
		self.ChannelSelect[25]:SetText(Resource[language][38]);
		self.ChannelSelect[26]:SetText(Resource[language][39]);
		self.ChannelSelect[27]:SetText(Resource[language][40]);
		self.SaveButton:SetText(Resource[language][41]);
		self.TestButton:SetText(Resource[language][42]);
		self.DeleteButton:SetText(Resource[language][43]);
	end
end
alertMain=Alerter();
